@@ -4,7 +4,7 @@ package DBIx::SearchBuilder;
 use strict;
 use warnings;
 
-our $VERSION = "1.45_02";
+our $VERSION = "1.55_01";
 
 use Clone qw();
 use Encode qw();
@@ -128,8 +128,6 @@ sub CleanSlate {
     my $self = shift;
     $self->RedoSearch();
     $self->{'itemscount'}       = 0;
-    $self->{'tables'}           = "";
-    $self->{'auxillary_tables'} = "";
     $self->{'where_clause'}     = "";
     $self->{'limit_clause'}     = "";
     $self->{'order'}            = "";
@@ -432,11 +430,19 @@ sub BuildSelectQuery {
       if ( $self->_isLimited > 0 );
 
     # DISTINCT query only required for multi-table selects
-    if ($self->_isJoined) {
+    # when we have group by clause then the result set is distinct as
+    # it must contain only columns we group by or results of aggregate
+    # functions which give one result per group, so we can skip DISTINCTing
+    if ( my $clause = $self->_GroupClause ) {
+        $QueryString = "SELECT main.* FROM $QueryString";
+        $QueryString .= $clause;
+        $QueryString .= $self->_OrderClause;
+    }
+    elsif ($self->_isJoined) {
         $self->_DistinctQuery(\$QueryString);
-    } else {
+    }
+    else {
         $QueryString = "SELECT main.* FROM $QueryString";
-        $QueryString .= $self->_GroupClause;
         $QueryString .= $self->_OrderClause;
     }
 
@@ -1082,19 +1088,20 @@ sub _OrderClause {
         else {
 	    $rowhash{'ORDER'} = "ASC";
         }
+        $rowhash{'ALIAS'} = 'main' unless defined $rowhash{'ALIAS'};
 
-        if ( ($rowhash{'ALIAS'}) and
-	     ($rowhash{'FIELD'}) and
-             ($rowhash{'ORDER'}) ) {
+        if ( defined $rowhash{'ALIAS'} and
+	     $rowhash{'FIELD'} and
+             $rowhash{'ORDER'} ) {
 
-	    if ($rowhash{'FIELD'} =~ /^(\w+\()(.*\))$/) {
+	    if ( length $rowhash{'ALIAS'} && $rowhash{'FIELD'} =~ /^(\w+\()(.*\))$/ ) {
 		# handle 'FUNCTION(FIELD)' formatted fields
 		$rowhash{'ALIAS'} = $1 . $rowhash{'ALIAS'};
 		$rowhash{'FIELD'} = $2;
 	    }
 
             $clause .= ($clause ? ", " : " ");
-            $clause .= $rowhash{'ALIAS'} . ".";
+            $clause .= $rowhash{'ALIAS'} . "." if length $rowhash{'ALIAS'};
             $clause .= $rowhash{'FIELD'} . " ";
             $clause .= $rowhash{'ORDER'};
         }
@@ -1234,19 +1241,25 @@ that left joins work, this method needs a TABLE for the second field
 rather than merely an alias.  For this type of join, it will return
 the alias generated by the join.
 
-Instead of ALIAS1/FIELD1, it's possible to specify EXPRESSION, to join ALIAS2/TABLE2 on an arbitrary expression.
+Instead of ALIAS1/FIELD1, it's possible to specify EXPRESSION, to join
+ALIAS2/TABLE2 on an arbitrary expression.
+
+It is also possible to join to a pre-existing, already-limited
+L<DBIx::SearchBuilder> object, by passing it as COLLECTION2, instead
+of providing an ALIAS2 or TABLE2.
 
 =cut
 
 sub Join {
     my $self = shift;
     my %args = (
-        TYPE   => 'normal',
-        FIELD1 => undef,
-        ALIAS1 => 'main',
-        TABLE2 => undef,
-        FIELD2 => undef,
-        ALIAS2 => undef,
+        TYPE        => 'normal',
+        FIELD1      => undef,
+        ALIAS1      => 'main',
+        TABLE2      => undef,
+        COLLECTION2 => undef,
+        FIELD2      => undef,
+        ALIAS2      => undef,
         @_
     );
 
@@ -1427,13 +1440,19 @@ sub CountAll {
 
     # If we haven't actually got all objects loaded in memory, we
     # really just want to do a quick count from the database.
-    if ( $self->{'must_redo_search'} || !$self->{'count_all'}) {
+    # or if we have paging enabled then we count as well and store it in count_all
+    if ( $self->{'must_redo_search'} || ( $self->RowsPerPage && !$self->{'count_all'} ) ) {
         # If we haven't already asked the database for the row count, do that
-        $self->_DoCount(1) unless ( $self->{'count_all'} );
+        $self->_DoCount(1);
 
         #Report back the raw # of rows in the database
         return ( $self->{'count_all'} );
     }
+    
+    # if we have paging enabled and have count_all then return it
+    elsif ( $self->RowsPerPage ) {
+        return ( $self->{'count_all'} );
+    }
 
     # If we have loaded everything from the DB we have an
     # accurate count already.
@@ -1546,21 +1565,7 @@ TODO: Why are they lowercased?
 =cut
 
 sub Fields {
-    my $self  = shift;
-    my $table = shift;
-
-    my $dbh = $self->_Handle->dbh;
-
-    # TODO: memoize this
-
-    return map lc( $_->[0] ), @{
-        eval {
-            $dbh->column_info( '', '', $table, '' )->fetchall_arrayref( [3] );
-          }
-          || $dbh->selectall_arrayref("DESCRIBE $table;")
-          || $dbh->selectall_arrayref("DESCRIBE \u$table;")
-          || []
-      };
+    return (shift)->_Handle->Fields( @_ );
 }
 
 
