@@ -2,13 +2,13 @@
 # Copyright 1999-2001 Jesse Vincent <jesse@fsck.com>
 
 package DBIx::SearchBuilder::Handle::Pg;
-use strict;
 
-use vars qw($VERSION @ISA $DBIHandle $DEBUG);
+use strict;
+use warnings;
+
 use base qw(DBIx::SearchBuilder::Handle);
-use Want qw(want howmany);
 
-use strict;
+use Want qw(howmany);
 
 =head1 NAME
 
@@ -38,21 +38,22 @@ it returns a database handle.
 sub Connect {
     my $self = shift;
     
-    $self->SUPER::Connect(@_);
+    my $rv = $self->SUPER::Connect(@_);
     $self->SimpleQuery("SET TIME ZONE 'GMT'");
     $self->SimpleQuery("SET DATESTYLE TO 'ISO'");
     $self->AutoCommit(1);
-    return ($DBIHandle); 
+    return ($rv); 
 }
 
 
 =head2 Insert
 
-Takes a table name as the first argument and assumes that the rest of the arguments
-are an array of key-value pairs to be inserted.
+Takes a table name as the first argument and assumes
+that the rest of the arguments are an array of key-value
+pairs to be inserted.
 
-In case of isnert failure, returns a Class::ReturnValue object preloaded
-with error info
+In case of insert failure, returns a L<Class::ReturnValue>
+object preloaded with error info.
 
 =cut
 
@@ -61,11 +62,9 @@ sub Insert {
     my $self  = shift;
     my $table = shift;
     my %args  = (@_);
-    my $sth   = $self->SUPER::Insert( $table, %args );
 
-    unless ($sth) {
-        return ($sth);
-    }
+    my $sth = $self->SUPER::Insert( $table, %args );
+    return $sth unless $sth;
 
     if ( $args{'id'} || $args{'Id'} ) {
         $self->{'id'} = $args{'id'} || $args{'Id'};
@@ -82,6 +81,19 @@ sub Insert {
     return ( $self->{'id'} );
 }
 
+=head2 InsertQueryString
+
+Postgres sepcific overriding method for
+L<DBIx::SearchBuilder::Handle/InsertQueryString>.
+
+=cut
+
+sub InsertQueryString {
+    my $self = shift;
+    my ($query_string, @bind) = $self->SUPER::InsertQueryString( @_ );
+    $query_string =~ s/\(\s*\)\s+VALUES\s+\(\s*\)\s*$/DEFAULT VALUES/;
+    return ($query_string, @bind);
+}
 
 =head2 IdSequenceName TABLE
 
@@ -102,7 +114,8 @@ sub IdSequenceName {
         # Regexp from DBIx::Class's Pg handle. Thanks to Marcus Ramberg
         if ( defined $foo->{'COLUMN_DEF'}
             && $foo->{'COLUMN_DEF'}
-            =~ m!^nextval\('"?([^"']+)"?'::(?:text|regclass)\)!i )
+            =~ m!^nextval\(+'"?([^"']+)"?'(::(?:text|regclass)\))+!i )
+
         {
             return $self->{'_sequences'}{$table} = $1;
         }
@@ -201,6 +214,7 @@ sub _MakeClauseCaseInsensitive {
     }
 }
 
+
 =head2 DistinctQuery STATEMENTREF
 
 takes an incomplete SQL SELECT statement and massages it to return a DISTINCT result set.
@@ -213,19 +227,23 @@ sub DistinctQuery {
     my $sb = shift;
     my $table = $sb->Table;
 
-    if ($sb->_OrderClause =~ /(?<!main)\./) {
-        # If we are ordering by something not in 'main', we need to GROUP
-        # BY and adjust the ORDER_BY accordingly
-        local $sb->{group_by} = [@{$sb->{group_by} || []}, {FIELD => 'id'}];
-        local $sb->{order_by} = [map {($_->{ALIAS} and $_->{ALIAS} ne "main") ? {%{$_}, FIELD => "min(".$_->{FIELD}.")"}: $_} @{$sb->{order_by}}];
-        my $group = $sb->_GroupClause;
-        my $order = $sb->_OrderClause;
-        $$statementref = "SELECT main.* FROM ( SELECT main.id FROM $$statementref $group $order ) distinctquery, $table main WHERE (main.id = distinctquery.id)";
-    } else {
-        $$statementref = "SELECT DISTINCT main.* FROM $$statementref";
-        $$statementref .= $sb->_GroupClause;
-        $$statementref .= $sb->_OrderClause;
-    }
+    return $self->SUPER::DistinctQuery( $statementref, $sb, @_ )
+        if $sb->_OrderClause !~ /(?<!main)\./;
+
+    # If we are ordering by something not in 'main', we need to GROUP
+    # BY and adjust the ORDER_BY accordingly
+    local $sb->{group_by} = [ map {+{FIELD => $_}} $self->Fields($table) ];
+    local $sb->{'order_by'} = [
+        map {
+            ($_->{'ALIAS'}||'') ne "main"
+            ? { %{$_}, FIELD => ((($_->{'ORDER'}||'') =~ /^des/i)?'MAX':'MIN') ."(".$_->{FIELD}.")" }
+            : $_
+        }
+        @{$sb->{'order_by'}}
+    ];
+    my $group = $sb->_GroupClause;
+    my $order = $sb->_OrderClause;
+    $$statementref = "SELECT main.* FROM $$statementref $group $order";
 }
 
 1;
