@@ -6,9 +6,9 @@ use warnings;
 
 use vars qw($AUTOLOAD);
 use Class::ReturnValue;
+use Encode qw();
 
 
-# {{{ Doc
 
 =head1 NAME
 
@@ -337,7 +337,7 @@ this HowTo to discuss using container classes,  overloading, and what
 ever else I think of.
 
 =head1 METHOD NAMING
- 
+
 Each method has a lower case alias; '_' is used to separate words.
 For example, the method C<_PrimaryKeys> has the alias C<_primary_keys>.
 
@@ -345,7 +345,6 @@ For example, the method C<_PrimaryKeys> has the alias C<_primary_keys>.
 
 =cut
 
-# }}}
 
 
 =head2  new 
@@ -366,7 +365,6 @@ sub new  {
     return $self;
   }
 
-# }}}
 
 # Not yet documented here.  Should almost certainly be overloaded.
 sub _Init {
@@ -375,7 +373,6 @@ sub _Init {
     $self->_Handle($handle);
 }
 
-# {{{ sub Id and id
 
 =head2 id
 
@@ -389,11 +386,9 @@ Returns this row's primary key.
 
 sub Id  {
     my $pkey = $_[0]->_PrimaryKey();
-    my $ret = $_[0]->{'values'}->{$pkey};
-    return $ret;
+    return $_[0]->{'values'}->{ $pkey };
 }
 
-# }}}
 
 =head2 primary_keys
 
@@ -408,21 +403,16 @@ Return a hash of the values of our primary keys for this function.
 
 sub PrimaryKeys { 
     my $self = shift; 
-    my %hash = map { $_ => $self->{'values'}->{$_} } @{$self->_PrimaryKeys};
-    return (%hash);
+    return map { $_ => $self->{'values'}->{$_} } @{$self->_PrimaryKeys};
 }
 
 
 
-# {{{ Routines dealing with getting and setting row data
 
-# {{{ sub DESTROY
 sub DESTROY {
     return 1;
 }
-# }}}
 
-# {{{ sub AUTOLOAD 
 
 sub AUTOLOAD {
     my $self = $_[0];
@@ -435,7 +425,7 @@ sub AUTOLOAD {
         goto &$AUTOLOAD;
     }
     elsif ( $self->_Accessible( $Attrib, 'record-read') ) {
-        *{$AUTOLOAD} = sub { $_[0]->_ToRecord( $Attrib, $_[0]->_Value($Attrib) ) };
+        *{$AUTOLOAD} = sub { $_[0]->_ToRecord( $Attrib, $_[0]->__Value($Attrib) ) };
         goto &$AUTOLOAD;        
     }
     elsif ( $self->_Accessible( $Attrib, 'foreign-collection') ) {
@@ -507,9 +497,7 @@ sub AUTOLOAD {
 
 }
 
-# }}}
 
-# {{{ sub _Accessible
 
 =head2 _Accessible KEY MODE
 
@@ -530,7 +518,6 @@ sub _Accessible {
     return $attribute->{$mode};
 }
 
-# }}}
 
 
 =head2 _PrimaryKeys
@@ -553,7 +540,6 @@ sub _PrimaryKey {
     return $pkeys->[0];
 }
 
-# {{{ sub _ClassAccessible
 
 =head2 _ClassAccessible 
 
@@ -594,9 +580,9 @@ sub _ClassAccessible {
 sub _ClassAccessibleFromSchema {
   my $self = shift;
   
-  my $accessible = {
-    # XXX TODO FIXME: should fetch custom primary key name
-    'id' => { 'read' => 1 },
+  my $accessible = {};
+  foreach my $key ($self->_PrimaryKeys) {
+   $accessible->{$key} = { 'read' => 1 };
   };
   
   my $schema = $self->Schema;
@@ -606,7 +592,12 @@ sub _ClassAccessibleFromSchema {
         $accessible->{$field} = { 'read' => 1, 'write' => 1 };
     } elsif (my $refclass = $schema->{$field}{'REFERENCES'}) {
         if (UNIVERSAL::isa($refclass, 'DBIx::SearchBuilder::Record')) {
-            $accessible->{$field} = { 'record-read' => 1, 'record-write' => 1 };
+            if ($field =~ /(.*)_id$/) {
+                $accessible->{$field} = { 'read' => 1, 'write' => 1 };
+                $accessible->{$1}     = { 'record-read' => 1, 'column' => $field };
+            } else {
+                $accessible->{$field} = { 'record-read' => 1, 'record-write' => 1 };
+            }
         } elsif (UNIVERSAL::isa($refclass, 'DBIx::SearchBuilder')) {
             $accessible->{$field} = { 'foreign-collection' => 1 };
         } else {
@@ -618,7 +609,6 @@ sub _ClassAccessibleFromSchema {
   return $accessible;  
 }
 
-# }}}
 
 sub _ToRecord {
     my $self = shift;
@@ -628,8 +618,10 @@ sub _ToRecord {
     return unless defined $value;
     
     my $schema = $self->Schema;
-    my $description = $schema->{$field};
+    my $description = $schema->{$field} || $schema->{$field . "_id"};
     
+    die "Can't get schema for $field on $self" unless $description;
+
     return unless $description;
     
     return $value unless $description->{'REFERENCES'};
@@ -676,13 +668,11 @@ Returns an array of the attributes of this class defined as "read" => 1 in this
 sub ReadableAttributes {
     my $self = shift;
     my $ca = $self->_ClassAccessible();
-    my @readable = grep { $ca->{$_}->{read}} keys %{$ca};
+    my @readable = grep { $ca->{$_}->{'read'} or $ca->{$_}->{'record-read'} } keys %{$ca};
     return (@readable);
 }
 
-# }}}
 
-# {{{  sub WritableAttributes 
 
 =head2 WritableAttributes
 
@@ -693,15 +683,12 @@ Returns an array of the attributes of this class defined as "write" => 1 in this
 sub WritableAttributes {
     my $self = shift;
     my $ca = $self->_ClassAccessible();
-    my @writable = grep { $ca->{$_}->{write}} keys %{$ca};
-    return (@writable);
-
+    my @writable = grep { $ca->{$_}->{'write'} || $ca->{$_}->{'record-write'} } keys %{$ca};
+    return @writable;
 }
 
-# }}}
 
 
-# {{{ sub __Value {
 
 =head2 __Value
 
@@ -712,26 +699,22 @@ override __Value.
 
 
 sub __Value {
-  my $self = shift;
-  my $field = lc shift;
+    my $self = shift;
+    my $field = lc shift;
 
-  if (!$self->{'fetched'}{$field} and my $id = $self->id() ) {
-    my $pkey = $self->_PrimaryKey();
-    my $QueryString = "SELECT $field FROM " . $self->Table . " WHERE $pkey = ?";
-    my $sth = $self->_Handle->SimpleQuery( $QueryString, $id );
-    my ($value) = eval { $sth->fetchrow_array() };
-    warn $@ if $@;
+    $field = $self->_Accessible($field, "column") if $self->_Accessible($field, "column");
 
-    $self->{'values'}{$field} = $value;
+    return $self->{'values'}{$field} if $self->{'fetched'}{$field};
     $self->{'fetched'}{$field} = 1;
-  }
-
-  my $value = $self->{'values'}{$field};
     
-  return $value;
+    my %pk = $self->PrimaryKeys;
+    return undef if grep !defined, values %pk;
+
+    my $query = "SELECT $field FROM ". $self->Table
+        ." WHERE ". join " AND ", map "$_ = ?", keys %pk;
+    my $sth = $self->_Handle->SimpleQuery( $query, values %pk ) or return undef;
+    return $self->{'values'}{$field} = ($sth->fetchrow_array)[0];
 }
-# }}}
-# {{{ sub _Value 
 
 =head2 _Value
 
@@ -746,9 +729,7 @@ sub _Value  {
   return ($self->__Value(@_));
 }
 
-# }}}
 
-# {{{ sub _Set 
 
 =head2 _Set
 
@@ -792,7 +773,17 @@ sub __Set {
         return ( $ret->return_value );
     }
     my $column = lc $args{'Column'};
-    if ( !defined( $args{'Value'} ) ) {
+
+    if ( defined $args{'Value'} ) {
+        if ( $args{'Value'} eq '' &&
+             ( $self->_Accessible( $args{'Column'}, 'is_numeric' )
+               || ($self->_Accessible( $args{'Column'}, 'type' ) || '') =~ /INT/i ) )
+        {
+            $args{'Value'} = 0;
+        }
+    }
+
+    unless ( defined $args{'Value'} ) {
         $ret->as_array( 0, "No value passed to _Set" );
         $ret->as_error(
             errno        => 2,
@@ -877,11 +868,60 @@ sub __Set {
     return ( $ret->return_value );
 }
 
-# }}}
+=head2 _Canonicalize PARAMHASH
+
+This routine massages an input value (VALUE) for FIELD into something that's 
+going to be acceptable.
+
+Takes
+
+=over
+
+=item FIELD
+
+=item VALUE
+
+=item FUNCTION
+
+=back
+
+
+Takes:
+
+=over
+
+=item FIELD
+
+=item VALUE
+
+=item FUNCTION
+
+=back
+
+Returns a replacement VALUE. 
+
+=cut
+
+sub _Canonicalize {
+    my $self = shift;
+    my $field = shift;
+    
+
+
+}
+
+
+=head2 _Validate FIELD VALUE
+
+Validate that VALUE will be an acceptable value for FIELD. 
+
+Currently, this routine does nothing whatsoever. 
+
+If it succeeds (which is always the case right now), returns true. Otherwise returns false.
+
+=cut
 
-# {{{ sub _Validate 
 
-#TODO: Implement _Validate.
 
 
 sub _Validate  {
@@ -900,9 +940,7 @@ sub _Validate  {
    return(1); 
   }	
 
-# }}}	
 
-# {{{ sub TruncateValue 
 
 =head2 TruncateValue  KEY VALUE
 
@@ -919,45 +957,37 @@ sub TruncateValue {
     my $value = shift;
 
     # We don't need to truncate empty things.
-    return undef unless (defined ($value));
+    return undef unless defined $value;
 
     my $metadata = $self->_ClassAccessible->{$key};
+    return $value unless $metadata;
 
     my $truncate_to;
     if ( $metadata->{'length'} && !$metadata->{'is_numeric'} ) {
-        $truncate_to = $metadata->{'length'};
+        $truncate_to = int $metadata->{'length'};
     }
-    elsif ($metadata->{'type'} &&  $metadata->{'type'} =~ /char\((\d+)\)/ ) {
+    elsif ($metadata->{'type'} && $metadata->{'type'} =~ /char\((\d+)\)/ ) {
         $truncate_to = $1;
     }
+    return $value unless $truncate_to;
 
-    return ($value) unless ($truncate_to);    # don't need to truncate
-
-    # Perl 5.6 didn't speak unicode
-    return substr( $value, 0, $truncate_to ) unless ( $] >= 5.007 );
-
-    require Encode;
+    # return asap if length in bytes is smaller than limit
+    return $value if $truncate_to >= do { use bytes; length $value };
 
     if ( Encode::is_utf8($value) ) {
-        return Encode::decode(
-            utf8 => substr( Encode::encode( utf8 => $value ), 0, $truncate_to ),
+        return Encode::decode_utf8(
+            substr( Encode::encode_utf8( $value ), 0, $truncate_to ),
             Encode::FB_QUIET(),
         );
     }
     else {
-        return Encode::encode(
-            utf8 => Encode::decode(
-                utf8 => substr( $value, 0, $truncate_to ),
-                Encode::FB_QUIET(),
-            )
-        );
-
+        # XXX: if it's not UTF-8 then why do we convert it to?
+        return Encode::encode_utf8( Encode::decode_utf8 (
+            substr( $value, 0, $truncate_to ), Encode::FB_QUIET(),
+        ) );
     }
-
 }
-# }}}
 
-# {{{ sub _Object 
 
 =head2 _Object
 
@@ -995,7 +1025,7 @@ sub __Object {
         eval "require $class";
         die "Couldn't use $class: $@" if ($@);
         unless ( $vglob && *$vglob{'SCALAR'} ) {
-            *{ $class . "::VERSION" } = '-1, By DBIx::SerchBuilder';
+            *{ $class . "::VERSION" } = '-1, By DBIx::SearchBuilder';
         }
     }
 
@@ -1004,11 +1034,8 @@ sub __Object {
     return $object;
 }
 
-# }}}
   
-# {{{ routines dealing with loading records
 
-# {{{ sub Load 
 
 # load should do a bit of overloading
 # if we call it with only one argument, we're trying to load by reference.
@@ -1025,14 +1052,11 @@ is $id
 
 
 
-sub Load  {
+sub Load {
     my $self = shift;
-    # my ($package, $filename, $line) = caller;
     return $self->LoadById(@_);
 }
 
-# }}}
-# {{{ sub LoadByCol 
 
 =head2 LoadByCol
 
@@ -1042,19 +1066,12 @@ undefined
 
 =cut
 
-
-
 sub LoadByCol  {
     my $self = shift;
-    my $col = shift;
-    my $val = shift;
-    
-    return($self->LoadByCols($col => $val));
+    return $self->LoadByCols(@_);
 }
 
-# }}}
 
-# {{{ sub LoadByCols
 
 =head2 LoadByCols
 
@@ -1110,9 +1127,7 @@ sub LoadByCols  {
 }
 
 
-# }}}
 
-# {{{ sub LoadById 
 
 =head2 LoadById
 
@@ -1122,18 +1137,12 @@ Loads a record by its primary key. Your record class must define a single primar
 
 
 sub LoadById  {
-    my $self = shift;
-    my $id = shift;
-
-    $id = 0 if (!defined($id));
-    my $pkey = $self->_PrimaryKey();
-    return ($self->LoadByCols($pkey => $id));
+    my ($self, $id) = @_;
+    return $self->LoadByCols( $self->_PrimaryKey, defined $id? $id: 0 );
 }
 
-# }}}  
 
 
-# {{{ LoadByPrimaryKeys 
 
 =head2 LoadByPrimaryKeys 
 
@@ -1155,10 +1164,8 @@ sub LoadByPrimaryKeys {
     return ($self->LoadByCols(%cols));
 }
 
-# }}}
 
 
-# {{{ sub LoadFromHash
 
 =head2 LoadFromHash
 
@@ -1181,9 +1188,7 @@ sub LoadFromHash {
   return $self->id();
 }
 
-# }}}
 
-# {{{ sub _LoadFromSQL 
 
 =head2 _LoadFromSQL QUERYSTRING @BIND_VALUES
 
@@ -1229,13 +1234,9 @@ sub _LoadFromSQL {
 
 }
 
-# }}}
 
-# }}}
 
-# {{{ Routines dealing with creating or deleting rows in the DB
 
-# {{{ sub Create 
 
 =head2 Create
 
@@ -1252,25 +1253,38 @@ sub Create {
 
     my ($key);
     foreach $key ( keys %attribs ) {
-        my $method = "Validate$key";
 
-        if ( $self->_Accessible( $key, 'record-write') ) {
-            $attribs{$key} = $attribs{$key}->id if UNIVERSAL::isa($attribs{$key}, 'DBIx::SearchBuilder::Record');
+        if ( $self->_Accessible( $key, 'record-write' ) ) {
+            $attribs{$key} = $attribs{$key}->id
+              if UNIVERSAL::isa( $attribs{$key},
+                'DBIx::SearchBuilder::Record' );
         }
 
-            #Truncate things that are too long for their datatypes
-        $attribs{$key} = $self->TruncateValue ($key => $attribs{$key});
-
-        unless ( $self->$method( $attribs{$key} ) ) {
-            delete $attribs{$key};
+        if ( defined $attribs{$key} ) {
+            if ( $attribs{$key} eq '' &&
+                 ( $self->_Accessible( $key, 'is_numeric' )
+                   || ($self->_Accessible( $key, 'type' ) || '') =~ /INT/i ) )
+            {
+                $attribs{$key} = 0;
+            }
         }
+        else {
+            $attribs{$key} = $self->_Accessible( $key, 'default' )
+                if $self->_Accessible( $key, 'no_nulls' );
+        }
+
+        #Truncate things that are too long for their datatypes
+        $attribs{$key} = $self->TruncateValue( $key => $attribs{$key} );
+
     }
     unless ( $self->_Handle->KnowsBLOBs ) {
+
         # Support for databases which don't deal with LOBs automatically
         my $ca = $self->_ClassAccessible();
         foreach $key ( keys %attribs ) {
             if ( $ca->{$key}->{'type'} =~ /^(text|longtext|clob|blob|lob)$/i ) {
-                my $bhash = $self->_Handle->BLOBParams( $key, $ca->{$key}->{'type'} );
+                my $bhash =
+                  $self->_Handle->BLOBParams( $key, $ca->{$key}->{'type'} );
                 $bhash->{'value'} = $attribs{$key};
                 $attribs{$key} = $bhash;
             }
@@ -1279,9 +1293,6 @@ sub Create {
     return ( $self->_Handle->Insert( $self->Table, %attribs ) );
 }
 
-# }}}
-
-# {{{ sub Delete 
 
 =head2 Delete
 
@@ -1321,12 +1332,9 @@ sub __Delete {
     } 
 }
 
-# }}}
-
-# }}}
 
 
-# {{{ sub Table
+
 
 =head2 Table
 
@@ -1344,9 +1352,7 @@ sub Table {
     return ($self->{'table'});
 }
 
-# }}}
 
-# {{{ sub _Handle 
 
 =head2 _Handle
 
@@ -1363,7 +1369,6 @@ sub _Handle  {
     return ($self->{'DBIxHandle'});
   }
 
-# }}}
 
 if( eval { require capitalization } ) {
 	capitalization->unimport( __PACKAGE__ );
@@ -1373,7 +1378,6 @@ if( eval { require capitalization } ) {
 
 __END__
 
-# {{{ POD
 
 
 =head1 AUTHOR
@@ -1390,5 +1394,4 @@ L<DBIx::SearchBuilder>
 
 =cut
 
-# }}}
 
